/**
 * This program shows the concept of queue manager backout.The message is returned to the
 * original queue if there is any problem faced by queue manager.
 * Creation date: (26/05/2001 11:44:06 AM)
 * @author: Upkar Sharma
 */
import java.io.*;
import com.ibm.mq.*;		// Include the MQSeries classes for Java package.


public class MQBackout {
	
	private String qManager ="QM_dell_116";	  // Define name of queue manager object to connect to.

	private MQQueueManager qMgr;
	
	public MQBackout() {
		try{
			
			// Create a connection to the queue manager
			qMgr =new MQQueueManager(qManager);

			// Set up the options on the queue we wish to open...
	  		// Note. All MQSeries Options are prefixed with MQC in Java.
			
	  		int openOptions =MQC.MQOO_INPUT_AS_Q_DEF |MQC.MQOO_OUTPUT ;
			
			

			// Now specify the queue that we wish to open,and the open options.
			// Here the queue name is "default".
			MQQueue system_default_local_queue = qMgr.accessQueue("default",
							openOptions);

			// Define a simple MQSeries message, and 
	   		//write some text in UTF format..
			MQMessage hello_world =new MQMessage();
			
			hello_world.writeUTF("Hello World");

			// specify the message options...
			MQPutMessageOptions pmo =new MQPutMessageOptions();
			// accept the defaults,
			// same as MQPMO_DEFAULT constant.

   			// put the message on the queue.
	
			system_default_local_queue.put(hello_world,pmo);
			System.out.println("Message put on queue");
			
			// Put the message for 10 seconds.View the message on the queue.			
			Thread.sleep(10000);
			
			// get the message back again...
	  			// First define a MQSeries message buffer to 
	  			//receive the message into..
	  			
			MQMessage retrievedMessage =new MQMessage();
			retrievedMessage.messageId =hello_world.messageId;

			// Set the get message options..
			
			MQGetMessageOptions gmo =new MQGetMessageOptions();  // accept the defaults
			
			gmo.options = MQC.MQGMO_SYNCPOINT; // same as MQGMO_DEFAULT 

			// get the message off the queue..
			
			system_default_local_queue.get(retrievedMessage,gmo);
			System.out.println("Message retrieved from the queue");
			
			// And prove we have the message by displaying the UTF message text
			String msgText =retrievedMessage.readUTF();
			
			System.out.println("The message is:"+msgText);

			// Close the queue.
			system_default_local_queue.close();
			
			Thread.sleep(10000);
			
			MQQueue newqueue = qMgr.accessQueue("Manisha", openOptions);
			
			MQMessage hello_world1 =new MQMessage();
			
			hello_world1.writeUTF(msgText);

			// specify the message options...
			//MQPutMessageOptions pmo =new MQPutMessageOptions();
			// accept the defaults,
			// same as MQPMO_DEFAULT constant.

   			// put the message on the queue.
	
			newqueue.put(hello_world1,pmo);
			System.out.println("Message put on queue");
			

			// Disconnect from the queue manager.
			qMgr.disconnect();	
			
			} // End of try
			
			// If an error has occurred in the above, try to identify what went wrong.
				// Was it an MQSeries error?
				
			catch (MQException ex) {
				System.out.println("An MQSeries error occurred :Completion code "+
				ex.completionCode +"Reason code "+ex.reasonCode);
				
				try {
				qMgr.backout();
				} catch(Exception e) {
					System.out.println("Queue Manager Backed out");
				}
				System.out.println("Queue Manager Backed out");
			}

			// Was it a Java buffer space error?
			
				catch (java.io.IOException ex) {
	
					System.out.println("An error occurred whilst writing to the message buffer: " + ex);
				}
			
			// Was it any other exception including java.lang.InterruptedException caused
			// by Thread.sleep(10000) method.
			
			catch (Exception ex) {
			
				System.out.println("An error occurred whilst writing to the message buffer:"+ex);
			} //End of catch.
	} // End of Constructor.
	public static void main(String args []){
		new MQBackout();
	}
} // End of Class.
